#****************************************************************
#
#                       ＭＥＡＴＥＳＴ
#                     www.meatest.com
#
#                  M631, M632, M641, M642
#
#****************************************************************

import sys
import socket
import time

# Configuration variables
IPaddr = '192.168.1.100'    # Please set the IPv4 address of one of your devices M631, M632, M641, M642 here.
tcpPort = 22                # Set TCP port here, default port for communication is 22.
test_resistance = 5         # Here please set the value of how many times to test, the script will test from 100Ω and repeatedly add 100Ω until the set value is reached.
LOW_TO = 2                  # Set communication timeout here, the recommended default value is 2.

class Calib:
    """ Manages communication with Meatest M631, M632, M641, M642 via ethernet using TCP sockets. """

    def __init__(self, calibIPaddr, tcpPort):  
        self.IPaddr = calibIPaddr
        self.tcpPort = tcpPort
        self.socket = None
        self.LOW_TO = LOW_TO
        if not self.openConnection():
            raise Exception("Device unreachable")
        self.initialized = True

    def openConnection(self):
        """ Initializes TCP connection and sets up open socket handler """
        try:
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.socket.settimeout(self.LOW_TO)
            self.socket.connect((self.IPaddr, self.tcpPort))
            print(f"Connected to {self.IPaddr}:{self.tcpPort}")
            return True
        except Exception as e:
            print(f"Failed to connect: {e}")
            return False

    def send_command(self, command):
        """ Sends a command to the M631, M632, M641, M642 device and returns the response. """
        if not self.initialized:
            raise Exception("Connection not initialized")
        try:
            if isinstance(command, str):
                command = command.encode('utf-8')
            print(f"Sending command: {command.decode('utf-8')}")
            self.socket.sendall(command + b'\n')
            response = self.receive_response()
            return response
        except Exception as e:
            print(f"Error sending command: {e}")
            return None

    def receive_response(self):
        """ Receives a response from the device. """
        response = b''
        while True:
            try:     
                response += self.socket.recv(1024)                    
            except socket.timeout:
                break
        return response.decode('utf-8')

    def clearStatus(self):
        """ Clears device status byte and error queue """
        scpiCmd = b'*CLS'
        self.send_command(scpiCmd)

    def waitCmdComplete(self):
        """ Sends *WAI: Prevents the device from processing further commands but DOES NOT HALT the PC program """
        scpiCmd = b'*WAI'
        self.send_command(scpiCmd)

    def isReady(self, opcTO):
        """ Sends *OPC?: Prevents the device from processing further commands and HALTS the PC program """
        scpiCmd = b'*OPC?'
        self.send_command(scpiCmd)

    def reset(self):
        """ Sends *RST to set the device to default state """
        scpiCmd = b'*RST'
        self.send_command(scpiCmd)

    def enableRemote(self):
        """ Sends SYST:REM to switch the device into Remote Mode (M631, M632, M641, M642 will respond only to *IDN? otherwise) """
        scpiCmd = b'SYST:REM'
        self.send_command(scpiCmd)

    def readErr(self):
        """ Sends SYST:ERR?: Reads device's error queue """
        scpiCmd = b'SYST:ERR?'
        error = self.send_command(scpiCmd)
        return error.strip() if error else None

    def identity(self):
        """ Sends *IDN?: Reads device info inc. Make, Model, SN, FW version """
        scpiCmd = b'*IDN?'
        idn = self.send_command(scpiCmd)
        return idn.strip() if idn else None

    def setResistance(self, resistance):
        """ Sets output resistance """
        scpiCmd = f'RES {resistance}'.encode('UTF-8')
        print(f"Setting resistance: {scpiCmd.decode('utf-8')}")
        response = self.send_command(scpiCmd)

    def outputOn(self):
        """ Sends OUTP:STAT 1 to turn the output on """
        scpiCmd = b'OUTP:STAT 1'
        self.send_command(scpiCmd)

    def outputOff(self):
        """ Sends OUTP:STAT 0 to turn the output off """
        scpiCmd = b'OUTP:STAT 0'
        self.send_command(scpiCmd)

    def close(self):
        """ Closes the socket connection. """
        if self.socket:
            self.socket.close()
            print("Connection closed.")

if __name__ == "__main__":
    # Connection initialization code block
    calibrator = Calib(IPaddr, tcpPort)
    calibrator.enableRemote()
    calibrator.reset()
    calibrator.isReady(0.5)
    calibrator.clearStatus()
    
    # This procedure prints device info, sets resistance, and prints errors along the way (if any)
    print(calibrator.identity())
    for resistance in range(100, (test_resistance * 100) + 1, 100):
        print(f"Attempting to set resistance to: {resistance}Ω")
        calibrator.setResistance(resistance)
        calibrator.outputOn()
        time.sleep(1)
        calibrator.outputOff()
        errors = calibrator.readErr()
        print(f"Error after setting resistance {resistance}Ω: {errors}")
        
    # Connection termination code block
    calibrator.close()
    print()
    input("Test completed successfully. Press Enter to exit.")
